//
//  MySQLStartupPref.m
//  MySQLStartup
//
//  Created by Alfredo Kojima on 1/4/05.
//  Copyright (c) 2005 MySQL AB. All rights reserved.
//

#include "mahelper.h"
#import "MySQLStartupPref.h"

#undef NSLocalizedString
#define NSLocalizedString(a,b) a

@implementation MySQLStartupPref

- init
{
  authRef= 0;
  curIsRunning= NO+10; // to force update
  return nil;
}

- (void) mainViewDidLoad
{
  [self isRunning];
  [self isAutoStarting];

  [NSTimer scheduledTimerWithTimeInterval: 2.0
                                   target:self
                                 selector:@selector(isRunning) 
                                 userInfo:nil 
                                  repeats:YES];

  [NSTimer scheduledTimerWithTimeInterval: 2.0 
                                   target:self
                                 selector:@selector(isAutoStarting) 
                                 userInfo:nil 
                                  repeats:YES];
}

- (void)dealloc
{
  if (authRef)
    AuthorizationFree(authRef,kAuthorizationFlagDefaults);
}


- (BOOL)isAutoStarting
{  
  BOOL autoStart;
  
  if (autoStart= mautoStartState())
    [autoStartCheck setState:NSOnState];
  else
    [autoStartCheck setState:NSOffState];
  
  return autoStart;
}


- (BOOL)isRunning
{
  NSTask *task = [[NSTask alloc] init];
  NSPipe *newPipe = [NSPipe pipe];
  NSFileHandle *readHandle = [newPipe fileHandleForReading];
  NSData *inData = nil;
  NSArray *array = nil;
  NSString *string = nil;
  BOOL state= NO;
  int i, count;
  NSBundle *bundle= [NSBundle bundleForClass:[self class]];
  
  [task setLaunchPath:@"/bin/ps"];
  [task setArguments:[NSArray arrayWithObjects:@"ax",@"-ocommand",nil]];
  [task setStandardOutput:newPipe];
  [task setStandardError:newPipe];
  [task launch];
  [task waitUntilExit];
  
  inData = [readHandle readDataToEndOfFile];
  string = [[NSString alloc] initWithData:inData encoding:NSASCIIStringEncoding];
  array = [string componentsSeparatedByString:@"\n"];
  [task release];
  
  count= [array count];
  for (i= 0; i < count; i++)
  {
    NSString *line= [array objectAtIndex:i];
    NSRange range;
    
    range= [line rangeOfString:@"mysqld"];
    if (range.location != NSNotFound)
    {
      state= YES;
      break;
    }
  }

  if (state != curIsRunning)
  {
    curIsRunning= state;
    if (state == YES) {
      [button setTitle:NSLocalizedString(@"Stop MySQL Server", nil)];
      [descrText setStringValue:@"The MySQL Database Server is started and ready for client connections.\n"
        "To shut the Server down, use the \"Stop MySQL Server\" button."];
      [warningText setHidden:NO];
      [[stateText cell] setTextColor: [NSColor greenColor]];
      [stateText setStringValue:@"running"];
      [stateImage setImage:
        [[[NSImage alloc] initWithContentsOfFile:[bundle pathForResource:@"instance_started"
                                                                 ofType:@"png"]] autorelease]];
    } else {
      [button setTitle:NSLocalizedString(@"Start MySQL Server", nil)];
      [descrText setStringValue:@"The MySQL Database Server is currently stopped.\n"
        "To start it, use the \"Start MySQL Server\" button."];
      [warningText setHidden:YES];
      [[stateText cell] setTextColor: [NSColor redColor]];
      [stateText setStringValue:@"stopped"];
      [stateImage setImage:
        [[[NSImage alloc] initWithContentsOfFile:[bundle pathForResource:@"instance_stopped"
                                                                  ofType:@"png"]] autorelease]];
    }
    [stateImage setNeedsDisplay];
  }
  [string release];
  
  return state;
}



- (int)performCommand:(MAHelperCommand)command
{
  //AuthorizationRef authRef= NULL;
  NSString *helperPath;
  AuthorizationItem right = { "com.mysql.administrator.helper", 0, NULL, 0 };
  AuthorizationRights rightSet = { 1, &right };
  OSStatus status;

  helperPath= [[NSBundle bundleForClass:[self class]] pathForResource:@"mahelper"
                                                               ofType:@""];

  if (!authRef)
  {
    if (AuthorizationCreate(NULL, kAuthorizationEmptyEnvironment, kAuthorizationFlagDefaults, 
                            &authRef)!=errAuthorizationSuccess)
    {
      NSLog(@"Could not create authorization reference object.");
      return MAHelperCommandInternalError;
    }
    
    status = AuthorizationCopyRights(authRef, &rightSet, kAuthorizationEmptyEnvironment, 
                                     kAuthorizationFlagDefaults | kAuthorizationFlagPreAuthorize
                                     | kAuthorizationFlagInteractionAllowed | kAuthorizationFlagExtendRights,
                                     NULL);
  } else
    status= errAuthorizationSuccess;
  if (status == errAuthorizationSuccess)
  {
    int hstatus;
    
    hstatus = mhelperPerformCommand(authRef, [helperPath UTF8String], command);
  
    return hstatus;
  }
  else
  {
    AuthorizationFree(authRef, kAuthorizationFlagDestroyRights);
    authRef= 0;
    if (status != errAuthorizationCanceled)
    {
      // pre-auth failed
      NSLog(@"Pre-auth failed");
      return MAHelperCommandAuthFailed;
    }
    else
      return MAHelperCommandCancelled;
  }
}


static NSString *errorMessage(int rc)
{
  switch (rc)
  {
    case MAHelperCommandInternalError:
      return @"Internal error.";
    case MAHelperCommandSuccess:
      return @"Success";
    case MAHelperCommandExecFailed:
      return @"Could not execute command.";
    case MAHelperCommandChildError:
      return @"Error in command.";
    case MAHelperCommandAuthFailed:
      return @"Authentication error.";
    case MAHelperCommandOperationFailed:
      return @"Operation failed.";
    case MAHelperCommandCancelled:
      return @"Cancelled.";
    case MAHelperCommandHelperNotFound:
      return @"Helper binary not found. Please reinstall the preference pane.";
    default:
      return [NSString stringWithFormat:@"%i", rc];//@"";
  }
}


- (IBAction)toggleServer:(id)sender
{
  MAHelperCommand command;
  BOOL start= ![self isRunning];
  int rc;

  command.authorizedCommandId= start ? MAHelperStartMySQL : MAHelperStopMySQL;
  
  if ((rc= [self performCommand:command])!=MAHelperCommandSuccess)
  {
    if (rc != MAHelperCommandCancelled)
    {
      if (start)
        NSRunAlertPanel(NSLocalizedString(@"Error",nil), 
                        [NSString stringWithFormat:NSLocalizedString(@"Could not startup MySQL Server.\nReason: %@",nil), errorMessage(rc)], 
                        nil, nil, nil);
      else
        NSRunAlertPanel(NSLocalizedString(@"Error",nil), 
                        [NSString stringWithFormat:NSLocalizedString(@"Could not stop MySQL Server.\nReason: %@",nil), errorMessage(rc)], 
                        nil, nil, nil);
    }
  }
}


- (IBAction)changeAutoStart:(id)sender
{
  MAHelperCommand command;
  BOOL flag= ![self isAutoStarting];
  int rc;
    
  command.authorizedCommandId= MAHelperToggleAutoStart;
  command.enable= flag;
  if ((rc = [self performCommand:command]) != MAHelperCommandSuccess)
  {
    if (rc != MAHelperCommandCancelled)
      NSRunAlertPanel(NSLocalizedString(@"Error",nil),
                      [NSString stringWithFormat:NSLocalizedString(@"Could not change automatic startup configuration.\nReason: %@",nil), errorMessage(rc)], 
                      nil, nil, nil);
  }
  [self isAutoStarting];
}


@end
