unit AdminServerConnections;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, ComCtrls, StdCtrls, ExtCtrls, InstanceSections, Menus,
  ApplicationDataModule, MySQLConnection, AuxFuncs,
  myx_public_interface, myx_admin_public_interface,
  PNGImage, MyxError, TntMenus, TntExtCtrls, TntStdCtrls,
  TntComCtrls, gnugettext;

type
  TAdminServerConnectionsForm = class(TInstanceSectionForm)
    ServerProcessesPnl: TTntPanel;
    BottomPnl: TTntPanel;
    UserPopupMenu: TTntPopupMenu;
    KillUserMI: TTntMenuItem;
    ServerProcessesPageControl: TTntPageControl;
    UserConnTabSheet: TTabSheet;
    ThreadsTabSheet: TTabSheet;
    ProcessPopupMenu: TTntPopupMenu;
    KillThreadMI: TTntMenuItem;
    ThreadsListView: TTntListView;
    Panel5: TTntPanel;
    ThreadsBevel: TTntBevel;
    Label8: TTntLabel;
    PageHeaderImg: TTntImage;
    UserConnPnl: TTntPanel;
    UserConnListView: TTntListView;
    UserConnThreadsListView: TTntListView;
    Splitter1: TTntSplitter ;
    Panel1: TTntPanel;
    UserConnBevel: TTntBevel;
    Label1: TTntLabel;
    PageHeader2Img: TTntImage;
    Label2: TTntLabel;
    Label3: TTntLabel;
    N1: TTntMenuItem;
    ShowUserInfoMI: TTntMenuItem;
    KillThreadBtn: TTntButton;
    KillUserBtn: TTntButton;
    RefreshBtn: TTntButton;
    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);

    procedure InitializeControls; override;

    procedure RefreshProcessList;
    procedure RefreshUserList;
    procedure GetProcessList(Sender: TObject);
    procedure RefreshBtnClick(Sender: TObject);
    procedure ServerProcessesPageControlChange(Sender: TObject);
    procedure UserConnListViewChange(Sender: TObject; Item: TListItem;
      Change: TItemChange);
    procedure UserConnThreadsListViewChange(Sender: TObject;
      Item: TListItem; Change: TItemChange);
    procedure ThreadsListViewChange(Sender: TObject; Item: TListItem;
      Change: TItemChange);
    procedure KillThreadBtnClick(Sender: TObject);
    procedure KillUserBtnClick(Sender: TObject);
    procedure UserPopupMenuPopup(Sender: TObject);

    procedure RefreshProcessLists(Sender: TObject);
    procedure ShowUserInfoMIClick(Sender: TObject);

    procedure ShowHideUserListViewColumns;

    procedure RefreshUserThreads(PrevID: WideString = '');

    procedure Disconnected(var Message: TMessage); message WM_Disconnected;
    procedure Reconnected(var Message: TMessage); message WM_Reconnected;
  private
    { Private declarations }
    process_list: TMYX_PROCESS_LIST;

    ShowUserNameAndDescription: Boolean;

    ServerConnectionsPNGImg: TPNGObject;
  public
    { Public declarations }

  end;

var
  AdminServerConnectionsForm: TAdminServerConnectionsForm;

implementation

{$R *.dfm}

uses
  PNGTools;
  
procedure TAdminServerConnectionsForm.FormCreate(Sender: TObject);
begin
  InitForm(self);
  
  DockedPanel:=ServerProcessesPnl;

  process_list:=nil;

  ServerProcessesPageControl.ActivePageIndex:=0;

  ServerConnectionsPNGImg:=LoadPNGImageFromResource('server_connections', PageHeaderImg);
  PageHeader2Img.Picture.Assign(ServerConnectionsPNGImg);

  ShowUserNameAndDescription:=False;

  ShowHideUserListViewColumns;

  if(Not(MySQLConn.Connected))then
  begin
    DisableEnableControls(ThreadsTabSheet, False);
    DisableEnableControls(UserConnTabSheet, False);

    DisableEnableControls(BottomPnl, False);
  end;
end;

procedure TAdminServerConnectionsForm.FormDestroy(Sender: TObject);
begin
  if(process_list<>nil)then
    process_list.Free;

  ServerConnectionsPNGImg.Free;
end;

procedure TAdminServerConnectionsForm.FormClose(Sender: TObject;
  var Action: TCloseAction);
begin
  //
end;

procedure TAdminServerConnectionsForm.InitializeControls;
begin
  RefreshProcessList;
end;

procedure TAdminServerConnectionsForm.GetProcessList(Sender: TObject);
var ProcessList: PMYX_PROCESS_LIST;
begin
  ProcessList:=myx_get_process_list(TFetchDataThread(Sender).Connection.MySQL);
  {if(ProcessList=nil)then
    ProcessList:=myx_get_process_list(TFetchDataThread(Sender).InstanceDM.PMySQL);}

  if(ProcessList=nil)then
    raise EMyxSQLError.Create(_('Could not fetch Process List.'),
      myx_mysql_errno(TFetchDataThread(Sender).Connection.MySQL),
      myx_mysql_error(TFetchDataThread(Sender).Connection.MySQL));

  process_list:=TMYX_PROCESS_LIST.create(ProcessList);
  myx_free_process_list(ProcessList);
end;

procedure TAdminServerConnectionsForm.RefreshProcessLists(Sender: TObject);
begin
  RefreshProcessList;
  RefreshUserList;

  if(ServerProcessesPageControl.ActivePage=ThreadsTabSheet)then
    KillThreadBtn.Enabled:=ThreadsListView.Selected<>nil
  else
    KillThreadBtn.Enabled:=UserConnThreadsListView.Selected<>nil;
end;

procedure TAdminServerConnectionsForm.RefreshProcessList;
var i: integer;
  Item: TTntListItem;
  PrevID: WideString;
begin
  if(Not(MySQLConn.Connected))then
    Exit;

  if(process_list=nil)then
    MySQLConn.FetchData(KindOfData_ProcessList,
      GetProcessList, RefreshProcessLists,
      nil, _('Fetching Process List ...'))
  else
  begin
    ThreadsListView.Items.BeginUpdate;
    try
      if(ThreadsListView.Selected<>nil)then
        PrevID:=ThreadsListView.Selected.Caption
      else
        PrevID:='';

      ThreadsListView.Clear;


      for i:=0 to process_list.process_infos.Count-1 do
      begin
        Item:=AddListViewItem(ThreadsListView, nil,
          process_list.process_infos[i].id, 22, process_list.process_infos[i]);
        Item.SubItems.Add(process_list.process_infos[i].user);
        Item.SubItemImages[0]:=14;
        Item.SubItems.Add(process_list.process_infos[i].host);
        Item.SubItems.Add(process_list.process_infos[i].db);
        Item.SubItems.Add(process_list.process_infos[i].command);
        Item.SubItems.Add(process_list.process_infos[i].time);
        Item.SubItems.Add(process_list.process_infos[i].state);
        Item.SubItems.Add(process_list.process_infos[i].info);

        if(PrevID=process_list.process_infos[i].id)then
        begin
          ThreadsListView.Selected:=Item;
        end;
      end;
    finally
      ThreadsListView.Items.EndUpdate;
    end;
  end;
end;

procedure TAdminServerConnectionsForm.RefreshUserList;
var i, j: integer;
  UserConnItem: TTntListItem;
  PUser: PMYX_USER;
  User: TMYX_USER;
  ConnectionCount: integer;
  user_name, description,
  PrevUser, PrevID: WideString;
begin
  if(process_list<>nil)then
  begin
    UserConnListView.Items.BeginUpdate;
    InitControls:=True;
    try
      if(UserConnListView.Selected<>nil)then
        PrevUser:=UserConnListView.Selected.Caption
      else
        PrevUser:='';

      if(UserConnThreadsListView.Selected<>nil)then
        PrevID:=UserConnThreadsListView.Selected.Caption
      else
        PrevID:='';

      UserConnListView.Clear;
      UserConnThreadsListView.Clear;

      for i:=0 to process_list.process_infos.Count-1 do
      begin
        //Look if the user has already been added to the list
        UserConnItem:=nil;
        for j:=0 to UserConnListView.Items.Count-1 do
          if(UserConnListView.Items[j].Caption=process_list.process_infos[i].user)then
          begin
            UserConnItem:=UserConnListView.Items[j];
            break;
          end;

        //If not, add the user
        if(UserConnItem=nil)then
        begin
          user_name:='';
          description:='';

          if(ShowUserNameAndDescription)then
          begin
            User:=nil;

            PUser:=myx_get_user(MySQLConn.MySQL,process_list.process_infos[i].user);

            //If there was a problem fetching the user info, deaktivate userinfo
            if(PUser=nil)then
            begin
              ShowUserNameAndDescription:=False;
              ShowHideUserListViewColumns;
            end
            else
            begin
              User:=TMYX_USER.create(PUser);
              try
                user_name:=User.user_name;
                description:=User.description;
              finally
                User.Free;
                myx_free_user(PUser);
              end;
            end;
          end
          else
            User:=nil;

          UserConnItem:=AddListViewItem(UserConnListView, nil,
            process_list.process_infos[i].user, 14, User);

          ConnectionCount:=0;
          for j:=0 to process_list.process_infos.Count-1 do
            if(process_list.process_infos[j].user=process_list.process_infos[i].user)then
            begin
              inc(ConnectionCount);
            end;

          UserConnItem.SubItems.Add(IntToStr(ConnectionCount));

          if(ShowUserNameAndDescription)then
          begin
            UserConnItem.SubItems.Add(user_name);
            UserConnItem.SubItems.Add(description);
          end;

          if(PrevUser=process_list.process_infos[i].user)then
          begin
            UserConnListView.Selected:=UserConnItem;

            RefreshUserThreads(PrevID);
          end;
        end;
      end;
    finally
      UserConnListView.Items.EndUpdate;
      InitControls:=False;
    end;
  end;
end;

procedure TAdminServerConnectionsForm.RefreshUserThreads(PrevID: WideString);
var i: integer;
  NewItem: TTntListItem;
begin
  KillUserBtn.Enabled:=(UserConnListView.Selected<>nil);

  UserConnThreadsListView.Items.BeginUpdate;
  try
    UserConnThreadsListView.Clear;

    if(UserConnListView.Selected<>nil)then
    begin
      for i:=0 to process_list.process_infos.Count-1 do
        if(UserConnListView.Selected.Caption=
          process_list.process_infos[i].user)then
        begin
          NewItem:=AddListViewItem(UserConnThreadsListView, nil,
            process_list.process_infos[i].id, 22, process_list.process_infos[i]);
          NewItem.SubItems.Add(process_list.process_infos[i].user);
          NewItem.SubItemImages[0]:=14;
          NewItem.SubItems.Add(process_list.process_infos[i].host);
          NewItem.SubItems.Add(process_list.process_infos[i].db);
          NewItem.SubItems.Add(process_list.process_infos[i].command);
          NewItem.SubItems.Add(process_list.process_infos[i].time);
          NewItem.SubItems.Add(process_list.process_infos[i].state);
          NewItem.SubItems.Add(process_list.process_infos[i].info);

          if(PrevID=process_list.process_infos[i].id)then
            UserConnThreadsListView.Selected:=NewItem;
        end;
    end;
  finally
    UserConnThreadsListView.Items.EndUpdate;
  end;
end;

procedure TAdminServerConnectionsForm.UserConnListViewChange(
  Sender: TObject; Item: TListItem; Change: TItemChange);
begin
  if(Not(InitControls))then
    RefreshUserThreads;
end;

procedure TAdminServerConnectionsForm.RefreshBtnClick(Sender: TObject);
begin
  //Clear current ProcessList
  if(process_list<>nil)then
  begin
    process_list.Free;
    process_list:=nil;
  end;

  //Get new one
  RefreshProcessList;
end;

procedure TAdminServerConnectionsForm.ServerProcessesPageControlChange(
  Sender: TObject);
begin
  if(ServerProcessesPageControl.ActivePage=ThreadsTabSheet)then
  begin
    KillThreadBtn.Enabled:=ThreadsListView.Selected<>nil;
    KillUserBtn.Enabled:=False;
  end
  else
  begin
    KillThreadBtn.Enabled:=UserConnThreadsListView.Selected<>nil;
    KillUserBtn.Enabled:=UserConnListView.Selected<>nil;
  end;
end;

procedure TAdminServerConnectionsForm.UserConnThreadsListViewChange(
  Sender: TObject; Item: TListItem; Change: TItemChange);
begin
  KillThreadBtn.Enabled:=(UserConnThreadsListView.Selected<>nil)
end;

procedure TAdminServerConnectionsForm.ThreadsListViewChange(
  Sender: TObject; Item: TListItem; Change: TItemChange);
begin
  KillThreadBtn.Enabled:=(ThreadsListView.Selected<>nil);
end;

procedure TAdminServerConnectionsForm.KillThreadBtnClick(Sender: TObject);
var i: integer;
  CurrentID: integer;
begin
  CurrentID:=myx_get_thread_id(MySQLConn.MySQL);

  if(ServerProcessesPageControl.ActivePage=ThreadsTabSheet)then
  begin
    for i:=0 to ThreadsListView.Items.Count-1 do
      if(ThreadsListView.Items[i].Selected)then
        if(CurrentID<>StrToInt(ThreadsListView.Items[i].Caption))then
          myx_kill_thread(MySQLConn.MySQL, StrToInt(ThreadsListView.Items[i].Caption));

  end
  else
    for i:=0 to UserConnThreadsListView.Items.Count-1 do
      if(UserConnThreadsListView.Items[i].Selected)then
        if(CurrentID<>StrToInt(UserConnThreadsListView.Items[i].Caption))then
          myx_kill_thread(MySQLConn.MySQL, StrToInt(UserConnThreadsListView.Items[i].Caption));

  RefreshBtnClick(self);
end;

procedure TAdminServerConnectionsForm.KillUserBtnClick(Sender: TObject);
var i, j: integer;
  CurrentID: integer;
begin
  if(UserConnListView.SelCount>0)then
  begin
    CurrentID:=myx_get_thread_id(MySQLConn.MySQL);

    for i:=0 to UserConnListView.Items.Count-1 do
      if(UserConnListView.Items[i].Selected)then
      begin
        for j:=0 to process_list.process_infos.Count-1 do
          if(process_list.process_infos[j].user=UserConnListView.Items[i].Caption)and
            (CurrentID<>StrToInt(process_list.process_infos[j].id))then
            myx_kill_thread(MySQLConn.MySQL, StrToInt(process_list.process_infos[j].id));
      end;
  end;
end;

procedure TAdminServerConnectionsForm.UserPopupMenuPopup(Sender: TObject);
begin
  KillUserMI.Enabled:=(UserConnListView.SelCount>0);
  ShowUserInfoMI.Checked:=ShowUserNameAndDescription;
end;

procedure TAdminServerConnectionsForm.ShowUserInfoMIClick(Sender: TObject);
begin
  ShowUserNameAndDescription:=Not(ShowUserNameAndDescription);

  ShowHideUserListViewColumns;

  RefreshUserList;
end;

procedure TAdminServerConnectionsForm.ShowHideUserListViewColumns;
var ListColumn: TListColumn;
begin
  if(Not(ShowUserNameAndDescription))then
  begin
    if(UserConnListView.Columns.Count>2)then
      UserConnListView.Columns.Delete(2);
    if(UserConnListView.Columns.Count>2)then
      UserConnListView.Columns.Delete(2);

    UserConnListView.Columns[1].AutoSize:=True;
  end
  else
  begin
    UserConnListView.Columns[1].AutoSize:=False;
    UserConnListView.Columns[1].Width:=80;

    ListColumn:=UserConnListView.Columns.Add;
    ListColumn.Caption:=_('Full Name');
    ListColumn.Width:=150;

    ListColumn:=UserConnListView.Columns.Add;
    ListColumn.Caption:=_('Description');
    ListColumn.Width:=135;
    ListColumn.AutoSize:=True;

    ListColumn.Width:=UserConnListView.Width-24-
      (UserConnListView.Columns[0].Width+
      UserConnListView.Columns[1].Width+
      UserConnListView.Columns[2].Width);
  end;
end;

procedure TAdminServerConnectionsForm.Disconnected(var Message: TMessage);
var i: integer;
begin
  for i:=0 to ServerProcessesPageControl.PageCount-1 do
    DisableEnableControls(ServerProcessesPageControl.Pages[i], False);

  DisableEnableControls(BottomPnl, False);
end;

procedure TAdminServerConnectionsForm.Reconnected(var Message: TMessage);
var i: integer;
begin
  for i:=0 to ServerProcessesPageControl.PageCount-1 do
    DisableEnableControls(ServerProcessesPageControl.Pages[i], True);

  DisableEnableControls(BottomPnl, True);

  RefreshBtnClick(self);
end;

end.
