unit OptionsEditor;

// Copyright (C) 2003, 2004 MySQL AB
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, TntForms,
  Dialogs, TntStdCtrls, TntExtCtrls, TntComCtrls, ComCtrls, ImgList, AuxFuncs,
  myx_public_interface, Grids, ValEdit,
  Options, PNGImage, MyxError, Forms, StdCtrls, ExtCtrls, TntClasses;

type
  TOptionsForm = class(TTntForm)
    HeaderPnl: TTntPanel;
    HeaderImg: TTntImage;
    TitleLbl: TTntLabel;
    OptionTreeImageList: TImageList;
    OptionPageControl: TTntPageControl;
    GeneralTabSheet: TTabSheet;
    ConnectionsTabSheet: TTabSheet;
    GroupBox1: TTntGroupBox;
    ShowTipOfDayCBox: TTntCheckbox;
    StoreWindowsPosCBox: TTntCheckbox;
    GroupBox2: TTntGroupBox;
    CoverPnl: TTntPanel;
    CloseBtn: TTntButton;
    Label2: TTntLabel;
    ApplicationFontBtn: TTntButton;
    ConnsTreeView: TTntTreeView;
    ConnectionPageControl: TTntPageControl;
    ConnGeneralTabSheet: TTabSheet;
    ConnAdvTabSheet: TTabSheet;
    ConnectionLbl: TTntLabel;
    UsernameLbl: TTntLabel;
    PasswordLbl: TTntLabel;
    PortLbl: TTntLabel;
    HostnameLbl: TTntLabel;
    TypeLbl: TTntLabel;
    SchemataEdLbl: TTntLabel;
    PasswordEd: TTntEdit;
    PortEd: TTntEdit;
    TypeCBox: TTntComboBox;
    SchemataEd: TTntEdit;
    UsernameEd: TTntEdit;
    ConnectionEd: TTntEdit;
    HostnameEd: TTntEdit;
    Label1: TTntLabel;
    ConnMemo: TTntMemo;
    ApplyChangesBtn: TTntButton;
    DiscardChangesBtn: TTntButton;
    ConnValueListEditor: TValueListEditor;
    Label5: TTntLabel;
    AddConnectionBtn: TTntButton;
    DeleteConnectionBtn: TTntButton;
    SidebarScrollBox: TTntScrollbox;
    Panel1: TTntPanel;
    EditorsTabSheet: TTabSheet;
    GroupBox4: TTntGroupBox;
    EdTblShowSQLBeforeApplyingCBox: TTntCheckbox;
    EdTblAllColsNotNullCBox: TTntCheckbox;
    Label6: TTntLabel;
    EdTblPKAutoNamingEd: TTntComboBox;
    EdTblAllIntUnsignedCBox: TTntCheckbox;
    Label7: TTntLabel;
    EdTblIndexNamingEd: TTntComboBox;
    EdTblFKNamingEd: TTntComboBox;
    Label8: TTntLabel;
    Label9: TTntLabel;
    EdTblPKDatatypeEd: TTntComboBox;
    Label10: TTntLabel;
    EdTblDefColumnDatatypeEd: TTntComboBox;
    HeaderBGShape: TTntShape;
    TntLabel1: TTntLabel;
    DataFontBtn: TTntButton;
    TntLabel2: TTntLabel;
    CodeFontBtn: TTntButton;
    Label3: TTntLabel;
    LanguageCBox: TTntComboBox;
    TntLabel3: TTntLabel;
    TntLabel4: TTntLabel;
    TntLabel5: TTntLabel;
    TntGroupBox1: TTntGroupBox;
    StorePasswordCBox: TTntCheckBox;
    PasswordStorageTypeLbl: TTntLabel;
    PasswordStorageTypeCBox: TTntComboBox;
    TntGroupBox2: TTntGroupBox;
    IgnoreWarningsList: TTntListBox;
    TntLabel6: TTntLabel;
    RemoveWarningFromIgnoreListBtn: TTntButton;
    TntLabel7: TTntLabel;
    DefaultFontLU: TTntComboBox;
    DefaultFontSizeLU: TTntComboBox;
    DataFontSizeLU: TTntComboBox;
    DataFontLU: TTntComboBox;
    CodeFontLU: TTntComboBox;
    CodeFontSizeLU: TTntComboBox;
    CodeFontWidthLU: TTntComboBox;
    DisableTransparencyEffectsCBox: TTntCheckBox;
    TntLabel8: TTntLabel;
    TntLabel9: TTntLabel;
    TntLabel10: TTntLabel;
    TntLabel11: TTntLabel;
    DefaultStorageEngineLU: TTntComboBox;

    constructor Create(AOwner: TComponent; ApplOptionForm: TApplicationOptionsForm); reintroduce;
    destructor Destroy; override;

    procedure FormCreate(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
    procedure FormClose(Sender: TObject; var Action: TCloseAction);

    procedure ShowOptionPage(PageNr: Integer);
    procedure OptionImgClick(Sender: TObject);
    procedure AddOptionPage(IconName: WideString; TopPos: Integer;
      PageNr: Integer; PageTitle: WideString);
    procedure BuildOptionSideBar;

    procedure CloseBtnClick(Sender: TObject);
    procedure OptionTreeViewChange(Sender: TObject; Node: TTreeNode);
    procedure OptionPageControlChange(Sender: TObject);

    procedure LoadPageData(PageNr: Integer);
    procedure DoPageContentChanged(Sender: TObject);
    procedure ApplyChanges;
    function DiscardChanges(ConfirmDiscard: Boolean = True): Boolean;

    procedure OptionTreeViewChanging(Sender: TObject; Node: TTreeNode;
      var AllowChange: Boolean);
    procedure ApplyChangesBtnClick(Sender: TObject);
    procedure DiscardChangesBtnClick(Sender: TObject);
    procedure ConnsTreeViewChanging(Sender: TObject; Node: TTreeNode;
      var AllowChange: Boolean);
    procedure DeleteConnectionBtnClick(Sender: TObject);

    procedure AddConnection(UserConn: TMYX_USER_CONNECTION = nil);
    procedure AddConnectionBtnClick(Sender: TObject);

    procedure StorePasswordCBoxClick(Sender: TObject);

    procedure DisableEnableConnectionPages(Enable: Boolean);
    procedure FormKeyDown(Sender: TObject; var Key: Word;
      Shift: TShiftState);
    procedure ConnsTreeViewChange(Sender: TObject; Node: TTreeNode);
    procedure ApplicationFontBtnClick(Sender: TObject);
    procedure DataFontBtnClick(Sender: TObject);
    procedure CodeFontBtnClick(Sender: TObject);
    procedure RemoveWarningFromIgnoreListBtnClick(Sender: TObject);
  private
    { Private declarations }
    PageContentChanged,
      InitControls: Boolean;

    NewConnection: TMYX_USER_CONNECTION;

    DiscardingChanges: Boolean;

    OptionsImgs: TList;

    ApplOptionForm: TApplicationOptionsForm;
  public
    { Public declarations }
    stored_conns: TMYX_USER_CONNECTIONS;
    last_connection: Integer;
  end;

const
  ConnectionsPage = -2;

//----------------------------------------------------------------------------------------------------------------------

implementation

{$R *.dfm}

uses
  PNGTools;
  
//----------------------------------------------------------------------------------------------------------------------

constructor TOptionsForm.Create(AOwner: TComponent; ApplOptionForm: TApplicationOptionsForm);

begin
  inherited Create(AOwner);

  self.ApplOptionForm := ApplOptionForm;
end;

//----------------------------------------------------------------------------------------------------------------------

destructor TOptionsForm.Destroy;

begin
  ApplOptionForm.Free;

  inherited Destroy;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TOptionsForm.FormCreate(Sender: TObject);

begin
  InitControls := True;
  try
    InitForm(self);

    if (not (MYXCommonOptions.RestoreWindowPos(self, False))) then
      Position := poMainFormCenter;

    stored_conns := nil;
    PageContentChanged := False;
    NewConnection := nil;
    last_connection := -1;

    OptionPageControl.ActivePageIndex := 0;
    ConnectionPageControl.ActivePageIndex := 0;

    OptionPageControl.Top := 20;
    OptionPageControl.Height := OptionPageControl.Height - 16;
    HeaderPnl.BringToFront;

    Height := Height - 16;

    PasswordEd.PasswordChar := '*';

    BuildOptionSideBar;

    ShowOptionPage(0);

    DisableEnableConnectionPages(False);
  finally
    InitControls := False;
  end;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TOptionsForm.FormDestroy(Sender: TObject);
var
  i: Integer;
begin
  MYXCommonOptions.AddWindowPos(self);

  for i := 0 to OptionsImgs.Count - 1 do
    TPNGObject(OptionsImgs[i]).Free;

  OptionsImgs.Free;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TOptionsForm.FormClose(Sender: TObject; var Action: TCloseAction);

begin
  if (not (DiscardChanges)) then
    Abort;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TOptionsForm.ShowOptionPage(PageNr: Integer);

var
  i: Integer;

begin
  if (not (DiscardChanges)) then
    Exit;

  if (PageNr < 0) then
    PageNr := OptionPageControl.PageCount + PageNr;

  OptionPageControl.ActivePageIndex := PageNr;
  TitleLbl.Caption := OptionPageControl.ActivePage.Caption;

  for i := 0 to SidebarScrollBox.ComponentCount - 1 do
    if (TComponent(SidebarScrollBox.Components[i]) is TTntLabel) then
    begin
      if (TComponent(SidebarScrollBox.Components[i]).Tag = PageNr) then
      begin
        TTntLabel(SidebarScrollBox.Components[i]).Color := $009F9F9F;
        TTntLabel(SidebarScrollBox.Components[i]).Transparent := False;
      end
      else
      begin
        TTntLabel(SidebarScrollBox.Components[i]).Transparent := True;
      end;
    end;

  OptionPageControlChange(self);
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TOptionsForm.OptionImgClick(Sender: TObject);

begin
  ShowOptionPage(TComponent(Sender).Tag);
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TOptionsForm.AddOptionPage(IconName: WideString; TopPos: Integer; PageNr: Integer; PageTitle: WideString);

var
  Img: TTntImage;
  Lbl: TTntLabel;
  PNGImg: TPNGObject;

begin
  Img := TTntImage.Create(SidebarScrollBox);
  Img.Parent := SidebarScrollBox;
  Img.Width := 48;
  Img.Height := 48;
  Img.Left := (SidebarScrollBox.Width - Img.Width - 2) div 2;
  Img.Top := TopPos;
  PNGImg := LoadPNGImageFromResource(IconName, Img);
  OptionsImgs.Add(PNGImg);
  Img.Tag := PageNr;
  Img.OnClick := OptionImgClick;

  Lbl := TTntLabel.Create(SidebarScrollBox);
  Lbl.Parent := SidebarScrollBox;
  Lbl.Caption := ' ' + PageTitle + ' ';
  Lbl.Left := (SidebarScrollBox.Width - Lbl.Width - 2) div 2;
  ;
  Lbl.Top := TopPos + 48 + 4;
  Lbl.Alignment := taCenter;
  Lbl.AutoSize := False;
  Lbl.Tag := PageNr;
  Lbl.Transparent := True;
  Lbl.OnClick := OptionImgClick;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TOptionsForm.BuildOptionSideBar;

var //Img: TTntImage;
  ypos, i: Integer;
  TabSheet: TTabSheet;
  ApplPagesCount: Integer;

begin
  OptionsImgs := TList.Create;

  //Create Background Img
  {if(ApplOptionForm<>nil)then
  begin
    Img:=TTntImage.Create(self);
    Img.Parent:=SidebarScrollBox;
    Img.Width:=111;
    Img.Height:=166;
    Img.Left:=0;
    Img.Top:=35+48+13;
    OptionsImgs.Add(LoadPNGImageFromResource('options_bg', Img));
  end;}

  //Start at 25
  ypos := 25;
  ApplPagesCount := 0;
  if (ApplOptionForm <> nil) then
  begin
    ApplOptionForm.DoPageContentChanged := DoPageContentChanged;

    if (ApplOptionForm.DockOptionPageControl <> nil) then
    begin
      ApplPagesCount := ApplOptionForm.DockOptionPageControl.PageCount;

      for i := 0 to ApplPagesCount - 1 do
      begin
        TabSheet := ApplOptionForm.DockOptionPageControl.Pages[0];
        TabSheet.PageControl := OptionPageControl;
        TabSheet.PageIndex := i;

        if (i < ApplOptionForm.OptionsImgNames.Count) then
          AddOptionPage(ApplOptionForm.OptionsImgNames[i],
            ypos, i, TabSheet.Caption);

        ypos := ypos + 84;
      end;
    end;
  end;

  AddOptionPage('options_general', ypos,
    ApplPagesCount, 'General Options');

  ypos := ypos + 84;
  AddOptionPage('options_connections', ypos,
    ApplPagesCount + 1, 'Connections');

  ypos := ypos + 84;
  AddOptionPage('options_editors', ypos,
    ApplPagesCount + 2, 'Editors');
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TOptionsForm.DisableEnableConnectionPages(Enable: Boolean);

var
  i: Integer;

begin
  //Enable/Disable Controls on pages
  for i := 0 to ConnectionPageControl.PageCount - 1 do
    DisableEnableControls(ConnectionPageControl.Pages[i], Enable);
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TOptionsForm.CloseBtnClick(Sender: TObject);

begin
  ModalResult := mrOK;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TOptionsForm.OptionTreeViewChange(Sender: TObject; Node: TTreeNode);

begin
  if (Node <> nil) then
    if (Node.Data <> nil) then
      if (TObject(Node.Data) is TTabSheet) then
      begin
        OptionPageControl.ActivePage := Node.Data;
        TitleLbl.Caption := Node.Text;

        OptionPageControlChange(self);
      end;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TOptionsForm.OptionPageControlChange(Sender: TObject);

begin
  LoadPageData(OptionPageControl.ActivePageIndex);
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TOptionsForm.ConnsTreeViewChange(Sender: TObject; Node: TTreeNode);

begin
  DeleteConnectionBtn.Enabled := False;

  if (Node <> nil) then
    if (Node.Data <> nil) then
    begin
      InitControls := True;
      try
        ConnectionEd.Text := TMYX_USER_CONNECTION(Node.Data).connection_name;
        UsernameEd.Text := TMYX_USER_CONNECTION(Node.Data).username;
        PasswordEd.Text := TMYX_USER_CONNECTION(Node.Data).password;
        HostnameEd.Text := TMYX_USER_CONNECTION(Node.Data).hostname;
        PortEd.Text := IntToStr(TMYX_USER_CONNECTION(Node.Data).port);
        TypeCBox.ItemIndex := Ord(TMYX_USER_CONNECTION(Node.Data).connection_type);
        SchemataEd.Text := TMYX_USER_CONNECTION(Node.Data).schema;
        ConnMemo.Text := TMYX_USER_CONNECTION(Node.Data).notes;

        ConnValueListEditor.Strings.Assign(
          TMYX_USER_CONNECTION(Node.Data).advanced_options);

        //Enable Controls on pages
        DisableEnableConnectionPages(True);
      finally
        InitControls := False;
      end;

      //Set last_connection
      last_connection := stored_conns.user_connections.IndexOf(TMYX_USER_CONNECTION(Node.Data));

      if (Visible) and (ConnectionEd.Enabled) and
        (ConnectionPageControl.ActivePage = ConnGeneralTabSheet) then
        ConnectionEd.SetFocus;

      DeleteConnectionBtn.Enabled := True;
    end
    else
    begin
      InitControls := True;
      try
        ConnectionEd.Text := '';
        UsernameEd.Text := '';
        PasswordEd.Text := '';
        HostnameEd.Text := '';
        PortEd.Text := '';
        TypeCBox.ItemIndex := -1;
        SchemataEd.Text := '';
        ConnMemo.Text := '';

        ConnValueListEditor.Strings.Clear;

        //Disable Controls on pages
        DisableEnableConnectionPages(False);
      finally
        InitControls := False;
      end;
    end;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TOptionsForm.DoPageContentChanged(Sender: TObject);

begin
  if (not (InitControls)) then
  begin
    PageContentChanged := True;

    ApplyChangesBtn.Enabled := True;
    DiscardChangesBtn.Enabled := True;
  end;
end;

//----------------------------------------------------------------------------------------------------------------------

function TOptionsForm.DiscardChanges(ConfirmDiscard: Boolean): Boolean;

var
  res: Integer;
  pos: Integer;

begin
  DiscardChanges := True;

  //Set DiscardingChanges so there will be no recursive calls
  DiscardingChanges := True;
  try
    if (PageContentChanged) then
    begin
      if (ConfirmDiscard) then
      begin
        res := ShowModalDialog('Save changes?',
          'Some of the settings have changed. Do you want to apply these ' +
          'changes permanently?', myx_mtConfirmation,
          'Yes'#13#10'No'#13#10'Cancel');

        if (res = 1) then
          ApplyChanges
        else
          if (res = 3) then
          begin
            DiscardChanges := False;
            Exit;
          end;
      end;

      if (OptionPageControl.ActivePage = ConnectionsTabSheet) then
      begin
        if (NewConnection <> nil) then
        begin
          try
            pos := stored_conns.user_connections.IndexOf(NewConnection);
            if (pos >= 0) then
              stored_conns.user_connections.Delete(pos);

            ConnsTreeView.Items.Delete(ConnsTreeView.Selected);
          finally
            NewConnection := nil;
          end;
        end
        else
          //Reset Connection Edits
          ConnsTreeViewChange(self, ConnsTreeView.Selected);
      end
      else
        LoadPageData(OptionPageControl.ActivePageIndex);

      PageContentChanged := False;
      ApplyChangesBtn.Enabled := False;
      DiscardChangesBtn.Enabled := False;
    end;
  finally
    DiscardingChanges := False;
  end;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TOptionsForm.OptionTreeViewChanging(Sender: TObject; Node: TTreeNode; var AllowChange: Boolean);

begin
  DiscardChanges;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TOptionsForm.ApplyChangesBtnClick(Sender: TObject);

begin
  ApplyChanges;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TOptionsForm.DiscardChangesBtnClick(Sender: TObject);

begin
  DiscardChanges(False);
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TOptionsForm.ConnsTreeViewChanging(Sender: TObject; Node: TTreeNode; var AllowChange: Boolean);

begin
  if (not (DiscardingChanges)) then
    AllowChange := DiscardChanges
  else
    AllowChange := True;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TOptionsForm.DeleteConnectionBtnClick(Sender: TObject);

var
  pos: Integer;
begin
  if (ConnsTreeView.Selected <> nil) and (stored_conns <> nil) then
    if (ConnsTreeView.Selected.Data <> nil) then
    begin
      pos := stored_conns.user_connections.IndexOf(
        TMYX_USER_CONNECTION(ConnsTreeView.Selected.Data));

      if (pos <> -1) then
      begin
        if (ShowModalDialog('Delete connection?',
          'Are you sure you want to delete the connection ' +
          TMYX_USER_CONNECTION(ConnsTreeView.Selected.Data).connection_name + '?',
          myx_mtConfirmation, 'Yes'#13#10'No') = 1) then
        begin
          PageContentChanged := False;
          ApplyChangesBtn.Enabled := False;
          DiscardChangesBtn.Enabled := False;

          stored_conns.user_connections.Delete(pos);
          ConnsTreeView.Items.Delete(ConnsTreeView.Selected);

          if (ConnsTreeView.Items.Count > 0) then
            ConnsTreeView.Selected := ConnsTreeView.Items[0];

          DoPageContentChanged(self);

          ApplyChanges;
        end;
      end;
    end;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TOptionsForm.AddConnection(UserConn: TMYX_USER_CONNECTION = nil);

var
  Node: TTntTreeNode;
  FreeConnection: Boolean;

begin
  if (UserConn = nil) then
  begin
    UserConn := TMYX_USER_CONNECTION.Create('New Connection',
      '', '', '',
      3306, '',
      '', '',
      MYX_MYSQL_CONN, MYX_FAVORITE_USER_CONNECTION);

    FreeConnection := True;
  end
  else
    FreeConnection := False;

  try
    Node := AddTreeViewChildNode(ConnsTreeView, ConnsTreeView.Items[0],
      'New Connection', 6,
      UserConn);

    stored_conns.user_connections.Add(UserConn);
    stored_conns.last_connection := stored_conns.user_connections.Count - 1;

    ConnsTreeView.Selected := Node;

    ConnectionEd.SelectAll;

    NewConnection := UserConn;

    DoPageContentChanged(self);
  except
    if (FreeConnection) then
      UserConn.Free;

    raise;
  end;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TOptionsForm.AddConnectionBtnClick(Sender: TObject);

begin
  AddConnection;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TOptionsForm.StorePasswordCBoxClick(Sender: TObject);

begin
  PasswordStorageTypeLbl.Enabled := StorePasswordCBox.Checked;
  PasswordStorageTypeCBox.Enabled := StorePasswordCBox.Checked;

  if (StorePasswordCBox.Checked) then
    PasswordStorageTypeCBox.ItemIndex := 0;

  DoPageContentChanged(Sender);
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TOptionsForm.LoadPageData(PageNr: Integer);

var
  I: Integer;
  user_conns: PMYX_USER_CONNECTIONS;
  error: MYX_LIB_ERROR;
  FavoritesNode,
  HistoryNode: TTntTreeNode;
  StrList: TTntStringList;
  FontSize: Single;

begin
  InitControls := True;
  try
    if (PageNr = OptionPageControl.PageCount - 3) then
    begin
      ShowTipOfDayCBox.Checked := MYXCommonOptions.ShowTipOfDay;
      StoreWindowsPosCBox.Checked := MYXCommonOptions.StoreWindowsPositions;
      DisableTransparencyEffectsCBox.Checked := MYXCommonOptions.DisableTransparencyEffects;

      LanguageCBox.Items.Text := MYXCommonOptions.AvailableLanguages;

      StrList := TTntStringList.Create;
      try
        StrList.Text := MYXCommonOptions.AvailableLanguageCodes;
        LanguageCBox.ItemIndex := StrList.IndexOf(MYXCommonOptions.Language);
      finally
        StrList.Free;
      end;

      StorePasswordCBox.Checked := not (
        (MYXCommonOptions.PasswordStorageType = MYX_PASSWORD_NOT_STORED));
      StorePasswordCBoxClick(self);

      PasswordStorageTypeCBox.Items.Text := MYXCommonOptions.AvailablePasswordStorageTypes;
      PasswordStorageTypeCBox.ItemIndex := Ord(MYXCommonOptions.PasswordStorageType) - 2;

      DefaultFontLU.Items.Assign(Screen.Fonts);
      DataFontLU.Items.Assign(Screen.Fonts);
      CodeFontLU.Items.Assign(Screen.Fonts);

      Screen.ResetFonts;

      DefaultFontLU.ItemIndex := DefaultFontLU.Items.IndexOf(
        MYXCommonOptions.DefaultFontName);
      if (DefaultFontLU.ItemIndex = -1) then
        DefaultFontLU.ItemIndex := DefaultFontLU.Items.IndexOf(
          'MS Sans Serif');

      // The font dialog only allows to pick fonts based on their font size, but we have its height here.
      // So we have to convert from height to size (assuming MM_TEXT as current DC mapping).
      FontSize := -MYXCommonOptions.DefaultFontHeight * 72 / GetDeviceCaps(Canvas.Handle, LOGPIXELSY);
      DefaultFontSizeLU.Text := FloatToStrF(FontSize, ffGeneral, 1, 3);

      DataFontLU.ItemIndex := DataFontLU.Items.IndexOf(
        MYXCommonOptions.DataFontName);
      if (DataFontLU.ItemIndex = -1) then
        DataFontLU.ItemIndex := DataFontLU.Items.IndexOf(
          'MS Sans Serif');

      FontSize := -MYXCommonOptions.DataFontHeight * 72 / GetDeviceCaps(Canvas.Handle, LOGPIXELSY);
      DataFontSizeLU.Text := FloatToStrF(FontSize, ffGeneral, 1, 3);

      CodeFontLU.ItemIndex := CodeFontLU.Items.IndexOf(
        MYXCommonOptions.CodeFontName);
      if (CodeFontLU.ItemIndex = -1) then
        CodeFontLU.ItemIndex := CodeFontLU.Items.IndexOf(
          'MS Sans Serif');

      CodeFontWidthLU.Text := IntToStr(MYXCommonOptions.CodeFontWidth);
      FontSize := -MYXCommonOptions.CodeFontHeight * 72 / GetDeviceCaps(Canvas.Handle, LOGPIXELSY);
      CodeFontSizeLU.Text := FloatToStrF(FontSize, ffGeneral, 1, 3);

      IgnoreWarningsList.Items.Text := MYXCommonOptions.IgnoreWarningsList.Text;
    end
    else
      if (PageNr = OptionPageControl.PageCount - 2) then
      begin
        if (stored_conns = nil) then
        begin
          ConnsTreeView.Items.Clear;

        //Fetch connections
          user_conns := myx_load_user_connections(
            MYXCommonOptions.UserDataDir + 'mysqlx_user_connections.xml', @error);
          if (error <> MYX_NO_ERROR) then
            raise EMyxError.Create('Error while loading stored connections.' +
              'Error Number ' + IntToStr(Ord(error)));
          try
            stored_conns := TMYX_USER_CONNECTIONS.create(user_conns);
          finally
            myx_free_user_connections(user_conns);
          end;

          FavoritesNode := AddTreeViewChildNode(ConnsTreeView, nil,
            'Connections', 5, nil);
          HistoryNode := AddTreeViewChildNode(ConnsTreeView, nil,
            'History', 5, nil);

          for i := 0 to stored_conns.user_connections.Count - 1 do
            if (stored_conns.user_connections[i].storage_type =
              MYX_FAVORITE_USER_CONNECTION) then
            begin
            //if(TMYX_USER_CONNECTION(stored_conns.user_connections[i]).storage_path='')then
              AddTreeViewChildNode(ConnsTreeView, FavoritesNode,
                stored_conns.user_connections[i].connection_name, 6,
                stored_conns.user_connections[i]);
            end
            else
            begin
              AddTreeViewChildNode(ConnsTreeView, HistoryNode,
                stored_conns.user_connections[i].username + '@' +
                stored_conns.user_connections[i].hostname, 6,
                stored_conns.user_connections[i]);
            end;

          FavoritesNode.Expand(True);
        end;
      end
      else
        if (PageNr = OptionPageControl.PageCount - 1) then
        begin
          EdTblShowSQLBeforeApplyingCBox.Checked := MYXCommonOptions.EditorTableShowSQLBeforeApplying;
          EdTblAllColsNotNullCBox.Checked := MYXCommonOptions.EditorTableAllColumnsNotNullPerDef;
          EdTblAllIntUnsignedCBox.Checked := MYXCommonOptions.EditorTableIntegerUnsignedPerDef;

          DefaultStorageEngineLU.Text := MYXCommonOptions.EditorTableDefaultStorageEngine;

          EdTblPKAutoNamingEd.Text := MYXCommonOptions.EditorTablePKAutoNaming;
          EdTblIndexNamingEd.Text := MYXCommonOptions.EditorTableIndexAutoNaming;
          EdTblFKNamingEd.Text := MYXCommonOptions.EditorTableFKAutoNaming;

          EdTblPKDatatypeEd.Text := MYXCommonOptions.EditorTablePKDataType;
          EdTblDefColumnDatatypeEd.Text := MYXCommonOptions.EditorTableDefColumnDataType;
        end
        else
          if (ApplOptionForm <> nil) then
            ApplOptionForm.SetControls(PageNr);
  finally
    InitControls := False;
  end;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TOptionsForm.ApplyChanges;

var
  StrList: TTntStringList;
  FontSize: Single;

begin
  if (OptionPageControl.ActivePage = GeneralTabSheet) then
  begin
    MYXCommonOptions.ShowTipOfDay := ShowTipOfDayCBox.Checked;
    MYXCommonOptions.StoreWindowsPositions := StoreWindowsPosCBox.Checked;
    MYXCommonOptions.DisableTransparencyEffects := DisableTransparencyEffectsCBox.Checked;

    StrList := TTntStringList.Create;
    try
      StrList.Text := MYXCommonOptions.AvailableLanguageCodes;
      MYXCommonOptions.Language := StrList[LanguageCBox.ItemIndex];
    finally
      StrList.Free;
    end;

    if (not (StorePasswordCBox.Checked)) then
      MYXCommonOptions.PasswordStorageType := MYX_PASSWORD_NOT_STORED
    else
      MYXCommonOptions.PasswordStorageType := MYX_PASSWORD_STORAGE_TYPE(PasswordStorageTypeCBox.ItemIndex + 2);

    MYXCommonOptions.DefaultFontName := DefaultFontLU.Text;

    // The font dialog only allows to pick fonts based on their font size, but we have its height here.
    // So we have to convert from height to size (assuming MM_TEXT as current DC mapping).
    FontSize := Abs(StrToFloatDef(DefaultFontSizeLU.Text, 11));
    MYXCommonOptions.DefaultFontHeight := -Round(FontSize * GetDeviceCaps(Canvas.Handle, LOGPIXELSY) / 72);

    MYXCommonOptions.DataFontName := DataFontLU.Text;
    FontSize := Abs(StrToFloatDef(DataFontSizeLU.Text, 11));
    MYXCommonOptions.DataFontHeight := -Round(FontSize * GetDeviceCaps(Canvas.Handle, LOGPIXELSY) / 72);

    MYXCommonOptions.CodeFontName := CodeFontLU.Text;
    MYXCommonOptions.CodeFontWidth := StrToIntDef(CodeFontWidthLU.Text, 8);
    FontSize := Abs(StrToFloatDef(CodeFontSizeLU.Text, 11));
    MYXCommonOptions.CodeFontHeight := -Round(FontSize * GetDeviceCaps(Canvas.Handle, LOGPIXELSY) / 72);

    MYXCommonOptions.IgnoreWarningsList.Text := IgnoreWarningsList.Items.Text;

    MYXCommonOptions.StoreOptions;
  end
  else
    if (OptionPageControl.ActivePage = ConnectionsTabSheet) then
    begin
      NewConnection := nil;

      if (ConnsTreeView.Selected <> nil) then
        if (ConnsTreeView.Selected.Data <> nil) then
        begin
          with TMYX_USER_CONNECTION(ConnsTreeView.Selected.Data) do
          begin
            connection_name := ConnectionEd.Text;
            username := UsernameEd.Text;
            password := PasswordEd.Text;
            hostname := HostnameEd.Text;
            port := StrToIntDef(PortEd.Text, 3306);
            connection_type := MYX_USER_CONNECTION_TYPE(TypeCBox.ItemIndex);
            schema := SchemataEd.Text;
            notes := ConnMemo.Text;

            advanced_options.Assign(ConnValueListEditor.Strings);
          end;

          ConnsTreeView.Selected.Text := ConnectionEd.Text;
        end;

      myx_store_user_connections(stored_conns.get_record_pointer,
        MYXCommonOptions.PasswordStorageType,
        MYXCommonOptions.UserDataDir + 'mysqlx_user_connections.xml');
    end
    else
      if (OptionPageControl.ActivePage = EditorsTabSheet) then
      begin
        MYXCommonOptions.EditorTableShowSQLBeforeApplying := EdTblShowSQLBeforeApplyingCBox.Checked;
        MYXCommonOptions.EditorTableAllColumnsNotNullPerDef := EdTblAllColsNotNullCBox.Checked;
        MYXCommonOptions.EditorTableIntegerUnsignedPerDef := EdTblAllIntUnsignedCBox.Checked;

        MYXCommonOptions.EditorTableDefaultStorageEngine := DefaultStorageEngineLU.Text;

        MYXCommonOptions.EditorTablePKAutoNaming := EdTblPKAutoNamingEd.Text;
        MYXCommonOptions.EditorTableIndexAutoNaming := EdTblIndexNamingEd.Text;
        MYXCommonOptions.EditorTableFKAutoNaming := EdTblFKNamingEd.Text;

        MYXCommonOptions.EditorTablePKDataType := EdTblPKDatatypeEd.Text;
        MYXCommonOptions.EditorTableDefColumnDataType := EdTblDefColumnDatatypeEd.Text;

        MYXCommonOptions.StoreOptions;
      end
      else
        if (ApplOptionForm <> nil) then
        begin
          ApplOptionForm.ApplyChanges(OptionPageControl.ActivePageIndex);

          MYXCommonOptions.StoreOptions;
        end;

  PageContentChanged := False;
  ApplyChangesBtn.Enabled := False;
  DiscardChangesBtn.Enabled := False;

  MessageToAllForms(WM_OptionsChanged, OptionPageControl.ActivePageIndex, 0);
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TOptionsForm.FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);

begin
  if (Key = VK_F1) then
    ShowHelp;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TOptionsForm.ApplicationFontBtnClick(Sender: TObject);

var
  FontDialog: TFontDialog;

begin
  FontDialog := TFontDialog.Create(nil);
  try
    FontDialog.Font.Name := DefaultFontLU.Text;
    FontDialog.Font.Size := StrToIntDef(DefaultFontSizeLU.Text, 8);
    if (FontDialog.Execute) then
    begin
      DefaultFontLU.Text := FontDialog.Font.Name;
      DefaultFontSizeLU.Text := IntToStr(FontDialog.Font.Size);
      DoPageContentChanged(Self);
    end;
  finally
    FontDialog.Free;
  end;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TOptionsForm.DataFontBtnClick(Sender: TObject);

var
  FontDialog: TFontDialog;

begin
  FontDialog := TFontDialog.Create(nil);
  try
    FontDialog.Font.Name := DataFontLU.Text;
    FontDialog.Font.Size := StrToIntDef(DataFontSizeLU.Text, 8);
    if (FontDialog.Execute) then
    begin
      DataFontLU.Text := FontDialog.Font.Name;
      DataFontSizeLU.Text := IntToStr(FontDialog.Font.Size);
      DoPageContentChanged(Self);
    end;
  finally
    FontDialog.Free;
  end;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TOptionsForm.CodeFontBtnClick(Sender: TObject);

var
  FontDialog: TFontDialog;

begin
  FontDialog := TFontDialog.Create(nil);
  try
    FontDialog.Font.Name := CodeFontLU.Text;
    FontDialog.Font.Size := StrToIntDef(CodeFontSizeLU.Text, 8);
    FontDialog.Font.Charset := MYXCommonOptions.CodeFontCharset;
    if (FontDialog.Execute) then
    begin
      CodeFontLU.Text := FontDialog.Font.Name;
      CodeFontSizeLU.Text := IntToStr(FontDialog.Font.Size);
      CodeFontWidthLU.Text := IntToStr(FontDialog.Font.Size);
      if (MYXCommonOptions.CodeFontCharset <> FontDialog.Font.Charset) then
        DoPageContentChanged(self);

      MYXCommonOptions.CodeFontCharset := FontDialog.Font.Charset;
      DoPageContentChanged(Self);
    end;
  finally
    FontDialog.Free;
  end;
end;

//----------------------------------------------------------------------------------------------------------------------

procedure TOptionsForm.RemoveWarningFromIgnoreListBtnClick(Sender: TObject);

var
  i: Integer;

begin
  i := 0;
  while (i < IgnoreWarningsList.Items.Count) do
  begin
    if (IgnoreWarningsList.Selected[i]) then
      IgnoreWarningsList.Items.Delete(i)
    else
      inc(i);
  end;

  DoPageContentChanged(self);
end;

//----------------------------------------------------------------------------------------------------------------------

end.

